@@ -1,4 +1,4 @@
-/* $Id: imap.c,v 1.23 2001/02/28 01:02:50 me Exp $
+/* $Id: imap.c,v 1.30 2001/11/16 21:23:13 me Exp $
  *
  * isync - IMAP4 to maildir mailbox synchronizer
  * Copyright (C) 2000-1 Michael R. Elkins <me@mutt.org>
@@ -278,7 +278,6 @@ buffer_gets (buffer_t * b, char **s)
 	    {
 		b->buf[b->offset] = 0;	/* terminate the string */
 		b->offset += 2;	/* next line */
-//              assert (strchr (*s, '\r') == 0);
 		return 0;
 	    }
 	}
@@ -421,7 +420,10 @@ imap_exec (imap_t * imap, const char *fmt, ...)
 
     snprintf (buf, sizeof (buf), "%d %s\r\n", ++Tag, tmp);
     if (Verbose)
+    {
+	fputs (">>> ", stdout);
 	fputs (buf, stdout);
+    }
     n = socket_write (imap->sock, buf, strlen (buf));
     if (n <= 0)
     {
@@ -556,7 +558,7 @@ imap_open (config_t * box, unsigned int minuid, imap_t * imap)
 {
     int ret;
     int s;
-    struct sockaddr_in sin;
+    struct sockaddr_in addr;
     struct hostent *he;
     int reuse = 0;
 #if HAVE_LIBSSL
@@ -609,9 +611,9 @@ imap_open (config_t * box, unsigned int minuid, imap_t * imap)
     {
 	/* open connection to IMAP server */
 
-	memset (&sin, 0, sizeof (sin));
-	sin.sin_port = htons (box->port);
-	sin.sin_family = AF_INET;
+	memset (&addr, 0, sizeof (addr));
+	addr.sin_port = htons (box->port);
+	addr.sin_family = AF_INET;
 
 	printf ("Resolving %s... ", box->host);
 	fflush (stdout);
@@ -623,14 +625,14 @@ imap_open (config_t * box, unsigned int minuid, imap_t * imap)
 	}
 	puts ("ok");
 
-	sin.sin_addr.s_addr = *((int *) he->h_addr_list[0]);
+	addr.sin_addr.s_addr = *((int *) he->h_addr_list[0]);
 
 	s = socket (PF_INET, SOCK_STREAM, 0);
 
-	printf ("Connecting to %s:%hu... ", inet_ntoa (sin.sin_addr),
-		ntohs (sin.sin_port));
+	printf ("Connecting to %s:%hu... ", inet_ntoa (addr.sin_addr),
+		ntohs (addr.sin_port));
 	fflush (stdout);
-	if (connect (s, (struct sockaddr *) &sin, sizeof (sin)))
+	if (connect (s, (struct sockaddr *) &addr, sizeof (addr)))
 	{
 	    perror ("connect");
 	    exit (1);
@@ -650,20 +652,22 @@ imap_open (config_t * box, unsigned int minuid, imap_t * imap)
 #if HAVE_LIBSSL
 	    if (box->use_imaps)
 		use_ssl = 1;
-	    else if (box->use_sslv2 || box->use_sslv3 || box->use_tlsv1)
+	    else
 	    {
 		/* let's see what this puppy can do... */
 		if ((ret = imap_exec (imap, "CAPABILITY")))
 		    break;
 
-		/* always try to select SSL support if available */
-		if (imap->have_starttls)
+		if (box->use_sslv2 || box->use_sslv3 || box->use_tlsv1)
 		{
-		    if ((ret = imap_exec (imap, "STARTTLS")))
-			break;
-		    use_ssl = 1;
+		    /* always try to select SSL support if available */
+		    if (imap->have_starttls)
+		    {
+			if ((ret = imap_exec (imap, "STARTTLS")))
+			    break;
+			use_ssl = 1;
+		    }
 		}
-
 	    }
 
 	    if (!use_ssl)
@@ -797,13 +801,16 @@ imap_open (config_t * box, unsigned int minuid, imap_t * imap)
 void
 imap_close (imap_t * imap)
 {
-    imap_exec (imap, "LOGOUT");
-    close (imap->sock->fd);
-    free (imap->sock);
-    free (imap->buf);
-    free_message (imap->msgs);
-    memset (imap, 0xff, sizeof (imap_t));
-    free (imap);
+    if (imap)
+    {
+	imap_exec (imap, "LOGOUT");
+	close (imap->sock->fd);
+	free (imap->sock);
+	free (imap->buf);
+	free_message (imap->msgs);
+	memset (imap, 0xff, sizeof (imap_t));
+	free (imap);
+    }
 }
 
 /* write a buffer stripping all \r bytes */
@@ -812,14 +819,29 @@ write_strip (int fd, char *buf, size_t len)
 {
     size_t start = 0;
     size_t end = 0;
+    ssize_t n;
 
     while (start < len)
     {
 	while (end < len && buf[end] != '\r')
 	    end++;
-	write (fd, buf + start, end - start);
-	end++;
-	start = end;
+	n = write (fd, buf + start, end - start);
+	if (n == -1)
+	{
+	    perror ("write");
+	    return -1;
+	}
+	else if ((size_t) n != end - start)
+	{
+	    /* short write, try again */
+	    start += n;
+	}
+	else
+	{
+	    /* write complete */
+	    end++;
+	    start = end;
+	}
     }
     return 0;
 }
@@ -877,7 +899,22 @@ imap_fetch_message (imap_t * imap, unsigned int uid, int fd)
 
 	    next_arg (&cmd);	/* * */
 	    next_arg (&cmd);	/* <msgno> */
-	    next_arg (&cmd);	/* FETCH */
+	    arg = next_arg (&cmd);	/* FETCH */
+
+	    if (strcasecmp ("FETCH", arg) != 0)
+	    {
+		/* this is likely an untagged response, such as when new
+		 * mail arrives in the middle of the session.  just skip
+		 * it for now.
+		 * 
+		 * eg.,
+		 * "* 4000 EXISTS"
+		 * "* 2 RECENT"
+		 *
+		 */
+		printf ("skipping untagged response: %s\n", arg);
+		continue;
+	    }
 
 	    while ((arg = next_arg (&cmd)) && *arg != '{')
 		;
@@ -887,7 +924,6 @@ imap_fetch_message (imap_t * imap, unsigned int uid, int fd)
 		return -1;
 	    }
 	    bytes = strtol (arg + 1, 0, 10);
-//          printf ("receiving %d byte message\n", bytes);
 
 	    /* dump whats left over in the input buffer */
 	    n = imap->buf->bytes - imap->buf->offset;
@@ -901,12 +937,14 @@ imap_fetch_message (imap_t * imap, unsigned int uid, int fd)
 	    /* ick.  we have to strip out the \r\n line endings, so
 	     * i can't just dump the raw bytes to disk.
 	     */
-	    write_strip (fd, imap->buf->buf + imap->buf->offset, n);
+	    if (write_strip (fd, imap->buf->buf + imap->buf->offset, n))
+	    {
+		/* write failed, message is not delivered */
+		return -1;
+	    }
 
 	    bytes -= n;
 
-//          printf ("wrote %d buffered bytes\n", n);
-
 	    /* mark that we used part of the buffer */
 	    imap->buf->offset += n;
 
@@ -919,8 +957,11 @@ imap_fetch_message (imap_t * imap, unsigned int uid, int fd)
 		n = socket_read (imap->sock, buf, n);
 		if (n > 0)
 		{
-//                  printf("imap_fetch_message:%d:read %d bytes\n", __LINE__, n);
-		    write_strip (fd, buf, n);
+		    if (write_strip (fd, buf, n))
+		    {
+			/* write failed */
+			return -1;
+		    }
 		    bytes -= n;
 		}
 		else
@@ -930,8 +971,6 @@ imap_fetch_message (imap_t * imap, unsigned int uid, int fd)
 		}
 	    }
 
-//          puts ("finished fetching msg");
-
 	    buffer_gets (imap->buf, &cmd);
 	    if (Verbose)
 		puts (cmd);	/* last part of line */
