@@ -1,4 +1,4 @@
-/* $Id: isync.h,v 1.14 2001/01/26 20:21:27 me Exp $
+/* $Id: isync.h,v 1.24 2001/11/20 18:06:09 me Exp $
  *
  * isync - IMAP4 to maildir mailbox synchronizer
  * Copyright (C) 2000 Michael R. Elkins <me@mutt.org>
@@ -23,6 +23,7 @@
 #if HAVE_LIBSSL
 #include <openssl/ssl.h>
 #endif
+#include "debug.h"
 
 typedef struct
 {
@@ -57,6 +58,7 @@ struct config
     char *box;
     char *alias;
     char *copy_deleted_to;
+    unsigned int max_messages;
     off_t max_size;
     config_t *next;
 #if HAVE_LIBSSL
@@ -70,6 +72,8 @@ struct config
 #endif
     unsigned int use_namespace:1;
     unsigned int expunge:1;
+    unsigned int delete:1;
+    unsigned int wanted:1;
 };
 
 /* struct representing local mailbox file */
@@ -98,12 +102,13 @@ struct message
     char *file;
     unsigned int uid;
     unsigned int flags;
-    unsigned int size;
+    size_t size;
     message_t *next;
     unsigned int processed:1;	/* message has already been evaluated */
     unsigned int new:1;		/* message is in the new/ subdir */
     unsigned int changed:1;	/* flags changed */
     unsigned int dead:1;	/* message doesn't exist on the server */
+    unsigned int wanted:1;	/* when using MaxMessages, keep this message */
 };
 
 /* struct used for parsing IMAP lists */
@@ -148,8 +153,14 @@ imap_t;
 /* flags for sync_mailbox */
 #define	SYNC_DELETE	(1<<0)	/* delete local that don't exist on server */
 #define SYNC_EXPUNGE	(1<<1)	/* don't fetch deleted messages */
+#define SYNC_QUIET	(1<<2)	/* only display critical errors */
+
+/* flags for maildir_open */
+#define OPEN_FAST	(1<<0)	/* fast open - don't parse */
+#define OPEN_CREATE	(1<<1)	/* create mailbox if nonexistent */
 
 extern config_t global;
+extern config_t *boxes;
 extern unsigned int Tag;
 extern char Hostname[256];
 extern int Verbose;
@@ -162,11 +173,12 @@ char *cram (const char *, const char *, const char *);
 
 char *next_arg (char **);
 
-int sync_mailbox (mailbox_t *, imap_t *, int, unsigned int);
+int sync_mailbox (mailbox_t *, imap_t *, int, unsigned int, unsigned int);
 
-void config_defaults (config_t *);
 void load_config (const char *);
+char * expand_strdup (const char *s);
 config_t *find_box (const char *);
+void free_config (void);
 
 void imap_close (imap_t *);
 int imap_copy_message (imap_t * imap, unsigned int uid, const char *mailbox);
@@ -176,11 +188,10 @@ int imap_expunge (imap_t *);
 imap_t *imap_open (config_t *, unsigned int, imap_t *);
 int imap_append_message (imap_t *, int, message_t *);
 
-mailbox_t *maildir_open (const char *, int fast);
+mailbox_t *maildir_open (const char *, int flags);
 int maildir_expunge (mailbox_t *, int);
-int maildir_sync (mailbox_t *);
 int maildir_set_uidvalidity (mailbox_t *, unsigned int uidvalidity);
-void maildir_close (mailbox_t *);
+int maildir_close (mailbox_t *);
 
 message_t * find_msg (message_t * list, unsigned int uid);
 void free_message (message_t *);
