@@ -1,4 +1,4 @@
-/* $Id: list.c,v 1.2 2001/01/24 07:09:28 me Exp $
+/* $Id: list.c,v 1.3 2001/11/19 19:41:14 me Exp $
  *
  * isync - IMAP4 to maildir mailbox synchronizer
  * Copyright (C) 2000-1 Michael R. Elkins <me@mutt.org>
@@ -153,7 +153,7 @@ free_list (list_t * list)
     {
 	tmp = list;
 	list = list->next;
-	if (is_list (list))
+	if (is_list (tmp))
 	    free_list (tmp->child);
 	else if (is_atom (tmp))
 	    free (tmp->val);
