@@ -1,4 +1,4 @@
-/* $Id: maildir.c,v 1.8 2001/02/19 18:44:15 me Exp $
+/* $Id: maildir.c,v 1.14 2001/11/20 18:06:09 me Exp $
  *
  * isync - IMAP4 to maildir mailbox synchronizer
  * Copyright (C) 2000-1 Michael R. Elkins <me@mutt.org>
@@ -27,6 +27,7 @@
 #include <unistd.h>
 #include <sys/stat.h>
 #include <errno.h>
+#include <time.h>
 #include "isync.h"
 
 static int
@@ -118,14 +119,16 @@ read_uid (const char *path, const char *file)
 
 }
 
-/* open a maildir mailbox.  if `fast' is nonzero, we just check to make
+/* open a maildir mailbox.
+ * if OPEN_FAST is set, we just check to make
  * sure its a valid mailbox and don't actually parse it.  any IMAP messages
  * with the \Recent flag set are guaranteed not to be in the mailbox yet,
  * so we can save a lot of time when the user just wants to fetch new messages
  * without syncing the flags.
+ * if OPEN_CREATE is set, we create the mailbox if it doesn't already exist.
  */
 mailbox_t *
-maildir_open (const char *path, int fast)
+maildir_open (const char *path, int flags)
 {
     char buf[_POSIX_PATH_MAX];
     DIR *d;
@@ -135,26 +138,72 @@ maildir_open (const char *path, int fast)
     mailbox_t *m;
     char *s;
     int count = 0;
-
-    /* check to make sure this looks like a valid maildir box */
-    snprintf (buf, sizeof (buf), "%s/new", path);
-    if (access (buf, F_OK))
-    {
-	perror ("access");
-	return 0;
-    }
-    snprintf (buf, sizeof (buf), "%s/cur", path);
-    if (access (buf, F_OK))
-    {
-	perror ("access");
-	return 0;
-    }
+    struct stat sb;
+    const char *subdirs[] = { "cur", "new", "tmp" };
+    int i;
 
     m = calloc (1, sizeof (mailbox_t));
-    m->path = strdup (path);
+    /* filename expansion happens here, not in the config parser */
+    m->path = expand_strdup (path);
+
+    if (stat (m->path, &sb))
+    {
+	if (errno == ENOENT && (flags & OPEN_CREATE))
+	{
+	    if (mkdir (m->path, S_IRUSR | S_IWUSR | S_IXUSR))
+	    {
+		fprintf (stderr, "ERROR: mkdir %s: %s (errno %d)\n",
+			 m->path, strerror (errno), errno);
+		free (m->path);
+		free (m);
+		return NULL;
+	    }
+
+	    for (i = 0; i < 3; i++)
+	    {
+		snprintf (buf, sizeof (buf), "%s/%s", m->path, subdirs[i]);
+		if (mkdir (buf, S_IRUSR | S_IWUSR | S_IXUSR))
+		{
+		    fprintf (stderr, "ERROR: mkdir %s: %s (errno %d)\n",
+			     buf, strerror (errno), errno);
+		    free (m->path);
+		    free (m);
+		    return NULL;
+		}
+	    }
+
+	}
+	else
+	{
+	    fprintf (stderr, "ERROR: stat %s: %s (errno %d)\n", m->path,
+		     strerror (errno), errno);
+	    free (m->path);
+	    free (m);
+	    return NULL;
+	}
+    }
+    else
+    {
+	/* check to make sure this looks like a valid maildir box */
+	for (i = 0; i < 3; i++)
+	{
+	    snprintf (buf, sizeof (buf), "%s/%s", m->path, subdirs[i]);
+	    if (stat (buf, &sb))
+	    {
+		fprintf (stderr, "ERROR: stat %s: %s (errno %d)\n", buf,
+			 strerror (errno), errno);
+		fprintf (stderr,
+			 "ERROR: %s does not appear to be a valid maildir style mailbox\n",
+			 m->path);
+		free (m->path);
+		free (m);
+		return 0;
+	    }
+	}
+    }
 
     /* check for the uidvalidity value */
-    m->uidvalidity = read_uid (path, "isyncuidvalidity");
+    m->uidvalidity = read_uid (m->path, "isyncuidvalidity");
     if (m->uidvalidity == (unsigned int) -1)
     {
 	free (m->path);
@@ -163,21 +212,21 @@ maildir_open (const char *path, int fast)
     }
 
     /* load the current maxuid */
-    if ((m->maxuid = read_uid (path, "isyncmaxuid")) == (unsigned int) -1)
+    if ((m->maxuid = read_uid (m->path, "isyncmaxuid")) == (unsigned int) -1)
     {
 	free (m->path);
 	free (m);
 	return NULL;
     }
 
-    if (fast)
+    if (flags & OPEN_FAST)
 	return m;
 
     cur = &m->msgs;
     for (; count < 2; count++)
     {
 	/* read the msgs from the new subdir */
-	snprintf (buf, sizeof (buf), "%s/%s", path,
+	snprintf (buf, sizeof (buf), "%s/%s", m->path,
 		  (count == 0) ? "new" : "cur");
 	d = opendir (buf);
 	if (!d)
@@ -218,11 +267,11 @@ maildir_open (const char *path, int fast)
 		    m->maxuidchanged = 1;
 		}
 		/* Courier-IMAP names it files
-		 * 	unique,S=<size>:info
+		 *      unique,S=<size>:info
 		 * so we need to put the UID before the size, hence here
 		 * we check for a comma as a valid terminator as well,
 		 * since the format will be
-		 * 	unique,U=<uid>,S=<size>:info
+		 *      unique,U=<uid>,S=<size>:info
 		 */
 		if (*s && *s != ':' && *s != ',')
 		{
@@ -337,8 +386,51 @@ update_maxuid (mailbox_t * mbox)
     return ret;
 }
 
+#define _24_HOURS (3600 * 24)
+
+static void
+maildir_clean_tmp (const char *mbox)
+{
+    char path[_POSIX_PATH_MAX];
+    DIR *dirp;
+    struct dirent *entry;
+    struct stat info;
+    time_t now;
+
+    snprintf (path, sizeof (path), "%s/tmp", mbox);
+    dirp = opendir (path);
+    if (dirp == NULL)
+    {
+	fprintf (stderr, "maildir_clean_tmp: opendir: %s: %s (errno %d)\n",
+		 path, strerror (errno), errno);
+	return;
+    }
+    /* assuming this scan will take less than a second, we only need to
+     * check the time once before the following loop.
+     */
+    time (&now);
+    while ((entry = readdir (dirp)))
+    {
+	snprintf (path, sizeof (path), "%s/tmp/%s", mbox, entry->d_name);
+	if (stat (path, &info))
+	    fprintf (stderr, "maildir_clean_tmp: stat: %s: %s (errno %d)\n",
+		     path, strerror (errno), errno);
+	else if (S_ISREG (info.st_mode) && now - info.st_ctime >= _24_HOURS)
+	{
+	    /* this should happen infrequently enough that it won't be
+	     * bothersome to the user to display when it occurs.
+	     */
+	    printf ("Warning: removing stale file %s\n", path);
+	    if (unlink (path))
+		fprintf (stderr,
+			 "maildir_clean_tmp: unlink: %s: %s (errno %d)\n",
+			 path, strerror (errno), errno);
+	}
+    }
+}
+
 int
-maildir_sync (mailbox_t * mbox)
+maildir_close (mailbox_t * mbox)
 {
     message_t *cur = mbox->msgs;
     char path[_POSIX_PATH_MAX];
@@ -380,6 +472,16 @@ maildir_sync (mailbox_t * mbox)
     if (mbox->maxuidchanged)
 	ret = update_maxuid (mbox);
 
+    /* per the maildir(5) specification, delivery agents are supposed to
+     * set a 24-hour timer on items placed in the `tmp' directory.
+     */
+    maildir_clean_tmp (mbox->path);
+
+    free (mbox->path);
+    free_message (mbox->msgs);
+    memset (mbox, 0xff, sizeof (mailbox_t));
+    free (mbox);
+
     return ret;
 }
 
@@ -421,12 +523,3 @@ maildir_set_uidvalidity (mailbox_t * mbox, unsigned int uidvalidity)
 
     return (ret);
 }
-
-void
-maildir_close (mailbox_t * mbox)
-{
-    free (mbox->path);
-    free_message (mbox->msgs);
-    memset (mbox, 0xff, sizeof (mailbox_t));
-    free (mbox);
-}
