@@ -1,4 +1,4 @@
-/* $Id: main.c,v 1.17 2001/02/14 20:46:41 me Exp $
+/* $Id: main.c,v 1.29 2001/11/20 18:06:09 me Exp $
  *
  * isync - IMAP4 to maildir mailbox synchronizer
  * Copyright (C) 2000-1 Michael R. Elkins <me@mutt.org>
@@ -33,7 +33,9 @@
 #include <getopt.h>
 
 struct option Opts[] = {
+    {"all", 0, NULL, 'a'},
     {"config", 1, NULL, 'c'},
+    {"create", 0, NULL, 'C'},
     {"delete", 0, NULL, 'd'},
     {"expunge", 0, NULL, 'e'},
     {"fast", 0, NULL, 'f'},
@@ -65,14 +67,13 @@ static void
 usage (void)
 {
     printf ("%s %s IMAP4 to maildir synchronizer\n", PACKAGE, VERSION);
-    puts ("Copyright (C) 2000 Michael R. Elkins <me@mutt.org>");
+    puts ("Copyright (C) 2000-1 Michael R. Elkins <me@mutt.org>");
     printf ("usage: %s [ flags ] mailbox [mailbox ...]\n", PACKAGE);
-    puts
-	("  -c, --config CONFIG	read an alternate config file (default: ~/.isyncrc)");
-    puts
-	("  -d, --delete		delete local msgs that don't exist on the server");
-    puts
-	("  -e, --expunge		expunge	deleted messages from the server");
+    puts ("  -a, --all	Synchronize all defined mailboxes");
+    puts ("  -c, --config CONFIG	read an alternate config file (default: ~/.isyncrc)");
+    puts ("  -C, --create		create local maildir mailbox if nonexistent");
+    puts ("  -d, --delete		delete local msgs that don't exist on the server");
+    puts ("  -e, --expunge		expunge	deleted messages from the server");
     puts ("  -f, --fast		only fetch new messages");
     puts ("  -h, --help		display this help message");
     puts ("  -p, --port PORT	server IMAP port");
@@ -80,8 +81,13 @@ usage (void)
     puts ("  -s, --host HOST	IMAP server address");
     puts ("  -u, --user USER	IMAP user name");
     puts ("  -v, --version		display version");
-    puts
-	("  -V, --verbose		verbose mode (display network traffic)");
+    puts ("  -V, --verbose		verbose mode (display network traffic)");
+    puts ("Compile time options:");
+#if HAVE_LIBSSL
+    puts ("  +HAVE_LIBSSL");
+#else
+    puts ("  -HAVE_LIBSSL");
+#endif
     exit (0);
 }
 
@@ -127,7 +133,7 @@ int
 main (int argc, char **argv)
 {
     int i;
-    config_t *box;
+    config_t *box = 0;
     mailbox_t *mail;
     imap_t *imap = 0;
     int expunge = 0;		/* by default, don't delete anything */
@@ -136,6 +142,8 @@ main (int argc, char **argv)
     char *config = 0;
     struct passwd *pw;
     int quiet = 0;
+    int all = 0;
+    int create = 0;
 
     pw = getpwuid (getuid ());
 
@@ -146,6 +154,7 @@ main (int argc, char **argv)
     global.user = strdup (pw->pw_name);
     global.maildir = strdup (pw->pw_dir);
     global.max_size = 0;
+    global.max_messages = 0;
     global.use_namespace = 1;
 #if HAVE_LIBSSL
     /* this will probably annoy people, but its the best default just in
@@ -157,56 +166,71 @@ main (int argc, char **argv)
     global.use_tlsv1 = 1;
 #endif
 
+#define FLAGS "aCc:defhp:qu:r:s:vV"
+
 #if HAVE_GETOPT_LONG
-    while ((i = getopt_long (argc, argv, "c:defhp:qu:r:s:vV", Opts, NULL)) != -1)
+    while ((i = getopt_long (argc, argv, FLAGS, Opts, NULL)) != -1)
 #else
-    while ((i = getopt (argc, argv, "c:defhp:u:qr:s:vV")) != -1)
+    while ((i = getopt (argc, argv, FLAGS)) != -1)
 #endif
     {
 	switch (i)
 	{
-	case 'c':
-	    config = optarg;
-	    break;
-	case 'd':
-	    delete = 1;
-	    break;
-	case 'e':
-	    expunge = 1;
-	    break;
-	case 'f':
-	    fast = 1;
-	    break;
-	case 'p':
-	    global.port = atoi (optarg);
-	    break;
-	case 'q':
-	    quiet = 1;
-	    Verbose = 0;
-	    break;
-	case 'r':
-	    global.box = optarg;
-	    break;
-	case 's':
-	    global.host = optarg;
-	    break;
-	case 'u':
-	    free (global.user);
-	    global.user = optarg;
-	    break;
-	case 'V':
-	    Verbose = 1;
-	    break;
-	case 'v':
-	    version ();
-	default:
-	    usage ();
+	    case 'a':
+		all = 1;
+		break;
+	    case 'C':
+		create = 1;
+		break;
+	    case 'c':
+		config = optarg;
+		break;
+	    case 'd':
+		delete = 1;
+		break;
+	    case 'e':
+		expunge = 1;
+		break;
+	    case 'f':
+		fast = 1;
+		break;
+	    case 'p':
+		global.port = atoi (optarg);
+		break;
+	    case 'q':
+		quiet = 1;
+		Verbose = 0;
+		break;
+	    case 'r':
+		global.box = optarg;
+		break;
+	    case 's':
+#if HAVE_LIBSSL
+		if (!strncasecmp ("imaps:", optarg, 6))
+		{
+		    global.use_imaps = 1;
+		    optarg += 6;
+		}
+#endif
+		global.host = optarg;
+		break;
+	    case 'u':
+		free (global.user);
+		global.user = optarg;
+		break;
+	    case 'V':
+		Verbose = 1;
+		break;
+	    case 'v':
+		version ();
+	    default:
+		usage ();
 	}
     }
 
-    if (!argv[optind])
+    if (!argv[optind] && !all)
     {
-	puts ("No box specified");
+	puts ("No mailbox specified");
 	usage ();
     }
 
@@ -214,21 +238,23 @@ main (int argc, char **argv)
 
     load_config (config);
 
-    for (; argv[optind]; optind++)
+    for (box = boxes; (all && box) || (!all && argv[optind]); optind++)
     {
-	box = find_box (argv[optind]);
-	if (!box)
+	if (!all)
 	{
-	    /* if enough info is given on the command line, don't worry if
-	     * the mailbox isn't defined.
-	     */
-	    if (!global.host)
+	    if (NULL == (box = find_box (argv[optind])))
 	    {
-		puts ("No such mailbox");
-		exit (1);
+		/* if enough info is given on the command line, don't worry if
+		 * the mailbox isn't defined.
+		 */
+		if (!global.host)
+		{
+		    fprintf (stderr, "%s: no such mailbox\n", argv[optind]);
+		    continue;
+		}
+		global.path = argv[optind];
+		box = &global;
 	    }
-	    global.path = argv[optind];
-	    box = &global;
 	}
 
 	if (!box->pass)
@@ -248,33 +274,46 @@ main (int argc, char **argv)
 	    box->pass = strdup (global.pass);
 	}
 
-	if(!quiet)
+	if (!quiet)
 	    printf ("Reading %s\n", box->path);
-	mail = maildir_open (box->path, fast);
+	i = 0;
+	if (fast)
+	    i |= OPEN_FAST;
+	if (create)
+	    i |= OPEN_CREATE;
+	mail = maildir_open (box->path, i);
 	if (!mail)
 	{
-	    puts ("Unable to load mailbox");
-	    exit (1);
+	    fprintf (stderr, "ERROR: unable to load mailbox %s\n", box->path);
+	    goto cleanup;
 	}
 
 	imap = imap_open (box, fast ? mail->maxuid + 1 : 1, imap);
 	if (!imap)
-	    exit (1);
+	{
+	    fprintf (stderr, "%s: skipping mailbox due to IMAP error\n",
+		     box->path);
+	    goto cleanup;
+	}
 
 	if (!quiet)
 	    puts ("Synchronizing");
-	i = delete ? SYNC_DELETE : 0;
+	i = 0;
+	if (quiet)
+	    i |= SYNC_QUIET;
+	i |= (delete || box->delete) ? SYNC_DELETE : 0;
 	i |= (expunge || box->expunge) ? SYNC_EXPUNGE : 0;
-	if (sync_mailbox (mail, imap, i, box->max_size))
+	if (sync_mailbox (mail, imap, i, box->max_size, box->max_messages))
 	    exit (1);
 
 	if (!fast)
 	{
-	    if (expunge && (imap->deleted || mail->deleted))
+	    if ((expunge || box->expunge) && (imap->deleted || mail->deleted))
 	    {
 		/* remove messages marked for deletion */
 		if (!quiet)
-		    printf ("Expunging %d messages from server\n", imap->deleted);
+		    printf ("Expunging %d messages from server\n",
+			    imap->deleted);
 		if (imap_expunge (imap))
 		    exit (1);
 		if (!quiet)
@@ -289,22 +328,28 @@ main (int argc, char **argv)
 	     */
 	    else if (delete)
 		maildir_expunge (mail, 1);
-
-	    /* write changed flags back to the mailbox */
-	    if (mail->changed)
-	    {
-		if (!quiet)
-		    printf ("Committing changes to %s\n", mail->path);
-		if (maildir_sync (mail))
-		    exit (1);
-	    }
 	}
 
-	maildir_close (mail);
+	/* write changed flags back to the mailbox */
+	if (!quiet)
+	    printf ("Committing changes to %s\n", mail->path);
+
+	if (maildir_close (mail))
+	    exit (1);
+
+      cleanup:
+	if (all)
+	    box = box->next;
     }
 
     /* gracefully close connection to the IMAP server */
     imap_close (imap);
 
+    free_config ();
+
+#if DEBUG
+    debug_cleanup ();
+#endif
+
     exit (0);
 }
