@@ -1,4 +1,4 @@
-/* $Id: sync.c,v 1.14 2001/02/14 20:46:41 me Exp $
+/* $Id: sync.c,v 1.20 2001/11/16 21:23:14 me Exp $
  *
  * isync - IMAP4 to maildir mailbox synchronizer
  * Copyright (C) 2000-1 Michael R. Elkins <me@mutt.org>
@@ -42,7 +42,7 @@ find_msg (message_t * list, unsigned int uid)
 
 int
 sync_mailbox (mailbox_t * mbox, imap_t * imap, int flags,
-	      unsigned int max_size)
+	      unsigned int max_size, unsigned int max_msgs)
 {
     message_t *cur;
     message_t *tmp;
@@ -53,6 +53,8 @@ sync_mailbox (mailbox_t * mbox, imap_t * imap, int flags,
     int fd;
     int ret;
     int fetched = 0;
+    int upload = 0;
+    unsigned int msg_count;
 
     if (mbox->uidvalidity > 0)
     {
@@ -91,9 +93,17 @@ sync_mailbox (mailbox_t * mbox, imap_t * imap, int flags,
 	    if (cur->uid == (unsigned int) -1)
 	    {
 		struct stat sb;
-		int fd;
 		int uid;
 
+		if ((flags & SYNC_QUIET) == 0)
+		{
+		    if (!upload)
+			fputs ("Uploading messages", stdout);
+		    fputc ('.', stdout);
+		    fflush (stdout);
+		    upload++;
+		}
+
 		/* upload the message if its not too big */
 		snprintf (path, sizeof (path), "%s/%s/%s", mbox->path,
 			  cur->new ? "new" : "cur", cur->file);
@@ -104,11 +114,13 @@ sync_mailbox (mailbox_t * mbox, imap_t * imap, int flags,
 
 		    continue;	/* not fatal */
 		}
-		if (sb.st_size > imap->box->max_size)
+		if (imap->box->max_size > 0
+		    && sb.st_size > imap->box->max_size)
 		{
-		    printf
-			("Warning, local message is too large (%ld), skipping...\n",
-			 sb.st_size);
+		    if ((flags & SYNC_QUIET) == 0)
+			printf
+			    ("Warning, local message is too large (%lu), skipping...\n",
+			     (unsigned long) sb.st_size);
 		    continue;
 		}
 		fd = open (path, O_RDONLY);
@@ -130,9 +142,6 @@ sync_mailbox (mailbox_t * mbox, imap_t * imap, int flags,
 		 */
 		if (uid != -1)
 		{
-		    char newpath[_POSIX_PATH_MAX];
-		    char *p;
-
 		    strfcpy (newpath, path, sizeof (newpath));
 		    /* kill :info field */
 		    p = strchr (newpath, ':');
@@ -153,17 +162,18 @@ sync_mailbox (mailbox_t * mbox, imap_t * imap, int flags,
 			perror ("rename");
 		}
 	    }
-	    else
+	    else if (flags & SYNC_DELETE)
 	    {
+		cur->flags |= D_DELETED;
+		cur->dead = 1;
+		mbox->deleted++;
+	    }
+	    /* if the user doesn't want local msgs deleted when they don't
+	     * exist on the server, warn that such messages exist.
+	     */
+	    else if ((flags & SYNC_QUIET) == 0)
 		printf ("Warning, uid %u doesn't exist on server\n",
 			cur->uid);
-		if (flags & SYNC_DELETE)
-		{
-		    cur->flags |= D_DELETED;
-		    cur->dead = 1;
-		    mbox->deleted++;
-		}
-	    }
 	    continue;
 	}
 	tmp->processed = 1;
@@ -203,9 +213,44 @@ sync_mailbox (mailbox_t * mbox, imap_t * imap, int flags,
 	}
     }
 
-    fputs ("Fetching new messages", stdout);
-    fflush (stdout);
-    for (cur = imap->msgs; cur; cur = cur->next)
+    if (upload)
+	fprintf (stdout, " %d messages.\n", upload);
+
+    if ((flags & SYNC_QUIET) == 0)
+    {
+	fputs ("Fetching new messages", stdout);
+	fflush (stdout);
+    }
+
+    if (max_msgs == 0)
+	max_msgs = UINT_MAX;
+    else
+    {
+	/* expire messages in excess of the max-count for this mailbox.
+	 * flagged mails are considered sacrosant and not deleted.
+	 * we have already done the upload to the server, so messing with
+	 * the flags variable do not have remote side effects.
+	 */
+	for (cur = imap->msgs, msg_count = 0;
+	     cur && msg_count < max_msgs; cur = cur->next, msg_count++)
+	{
+	    tmp = find_msg (mbox->msgs, cur->uid);
+	    if (tmp)
+		tmp->wanted = 1;
+	}
+	for (cur = mbox->msgs; cur; cur = cur->next)
+	{
+	    if (!cur->wanted && !(cur->flags & D_FLAGGED))
+	    {
+		cur->flags |= D_DELETED;
+		cur->dead = 1;
+		mbox->deleted++;
+	    }
+	}
+    }
+
+    for (cur = imap->msgs, msg_count = 0;
+	 cur && msg_count < max_msgs; cur = cur->next, msg_count++)
     {
 	if (!cur->processed)
 	{
@@ -222,9 +267,10 @@ sync_mailbox (mailbox_t * mbox, imap_t * imap, int flags,
 
 	    if (max_size && cur->size > max_size)
 	    {
-		printf
-		    ("Warning, message skipped because it is too big (%u)\n",
-		     cur->size);
+		if ((flags & SYNC_QUIET) == 0)
+		    printf
+			("Warning, message skipped because it is too big (%u)\n",
+			 cur->size);
 		continue;
 	    }
 
@@ -243,9 +289,9 @@ sync_mailbox (mailbox_t * mbox, imap_t * imap, int flags,
 	    for (;;)
 	    {
 		/* create new file */
-		snprintf (path, sizeof (path), "%s/tmp/%s.%ld_%d.%d,U=%d%s",
-			  mbox->path, Hostname, time (0), MaildirCount++,
-			  getpid (), cur->uid, suffix);
+		snprintf (path, sizeof (path), "%s/tmp/%ld_%d.%d.%s,U=%d%s",
+			  mbox->path, time (0), MaildirCount++, getpid (),
+			  Hostname, cur->uid, suffix);
 
 		if ((fd = open (path, O_WRONLY | O_CREAT | O_EXCL, 0600)) > 0)
 		    break;
@@ -261,14 +307,22 @@ sync_mailbox (mailbox_t * mbox, imap_t * imap, int flags,
 	    if (fd < 0)
 		continue;
 
-	    /* give some visual feedback that something is happening */
-	    fputs (".", stdout);
-	    fflush (stdout);
+	    if ((flags & SYNC_QUIET) == 0)
+	    {
+		/* give some visual feedback that something is happening */
+		fputs (".", stdout);
+		fflush (stdout);
+	    }
 	    fetched++;
 
 	    ret = imap_fetch_message (imap, cur->uid, fd);
 
-	    if (close (fd))
+	    if (fsync (fd))
+	    {
+		perror ("fsync");
+		close (fd);
+	    }
+	    else if (close (fd))
 		perror ("close");
 	    else if (!ret)
 	    {
@@ -288,7 +342,9 @@ sync_mailbox (mailbox_t * mbox, imap_t * imap, int flags,
 	    unlink (path);
 	}
     }
-    printf ("  %d messages\n", fetched);
+
+    if ((flags & SYNC_QUIET) == 0)
+	printf ("  %d messages\n", fetched);
 
     return 0;
 }
